#include "arch.h"
#include "exec.h"
#include "extr.h"
#include "mem.h"
#include "msg.h"
#include "opts.h"
#include "str.h"

#include <stdbool.h>
#include <stdlib.h>


const unsigned char Tries = 5;  /* values 1 - 9 are allowed */
bool Absolute, Keep, Unlink, Verbose;
char *Extension, **Archives;


static bool setup_opts(char **argv)
	{
	static const struct option opts[] =
		{
		'a', 0, &Absolute,
		'e', 1, &Extension,
		'k', 0, &Keep,
		'u', 0, &Unlink,
		'v', 0, &Verbose,
		0
		};

	Archives = parse_opts(argv, opts);
	return (Archives && *Archives) || (WRITE_STR(2, MSG_USAGE), false);
	}

#define TOGGLE(var) (var ? set_env(#var "=") : (unset_env(#var), true))

static bool setup_env(void)
	{
	/*
	 * Setting POSIXLY_CORRECT prevents GNU getopt (possibly used by external
	 * programs) from hallucinating options after non-option arguments.
	 */

	return TOGGLE(Verbose) && TOGGLE(Keep) && set_env("POSIXLY_CORRECT=");
	}

static bool setup_extrs(void)
	{
	char *home, *local;

	/* last added extractors enjoy priority (extractor list is a FIFO) */
	add_extrs(SHARE);
	home = getenv("HOME");
	if (home)
		{
		local = alloc(strlen(home) + sizeof "/.deco");
		str_cp(str_cp(local, home), "/.deco");
		add_extrs(local);
		free(local);
		}
	return Extractors || err(MSG_NO_EXTRS, false, NULL);
	}

int main(int argc, char **argv)
	{
	bool success = true;

	if (!setup_opts(argv) || !setup_env() || !setup_extrs())
		exit(EXIT_FAILURE);
	for (; *Archives; ++Archives)
		success = extract(*Archives) && success;
	exit(success ? EXIT_SUCCESS : EXIT_FAILURE);
	}
