#include "msg.h"

#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>


extern char **environ;


bool set_env(const char *var)
	{
	return putenv((char *)var) == 0 || err(MSG_PUTENV, true, var);
	}

void unset_env(const char *var)
	{
	size_t lvar;
	char **e;
	bool inc = 0;

	lvar = strlen(var);
	for (e = environ; *e; ++e)
		{
		inc = inc || (strncmp(*e, var, lvar) == 0 && (*e)[lvar] == '=');
		*e = *(e + inc);
		}
	}

bool spawn(void (*run)(void *), void *args, const char *msg, const char *id)
	{
	pid_t pid;
	int status;

	pid = fork();
	if (pid < 0)
		die(MSG_FORK, true, NULL);
	if (pid == 0)
		run(args);
	waitpid(pid, &status, 0);
	return WEXITSTATUS(status) == 0 || err(msg, false, id);
	}

void exec(void *args)
	{
	char **cmd = args;

	if (dup2(2, 1) != 1)
		die(MSG_STDIN_TO_STDERR, true, NULL);
	execvp(cmd[0], cmd);
	die(MSG_EXEC, true, cmd[0]);
	}
