#include "arch.h"
#include "deco.h"
#include "extr.h"
#include "fs.h"
#include "mem.h"
#include "msg.h"
#include "str.h"

#include <dirent.h>
#include <regex.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>


struct extractor *Extractors;


regoff_t match_extr(struct extractor *x, const char *file)
	{
	regmatch_t m;

	return (regexec(&x->matcher, file, 1, &m, 0) == 0
	        && m.rm_so > 0
	        && file[--m.rm_so] == '.') ? m.rm_so : -1;
	}

void find_extr(struct archive *a)
	{
	/* caller ensures a->extr == NULL */

	regoff_t pos;
	struct extractor *x;

	for (x = Extractors; x; x = x->next)
		{
		pos = match_extr(x, a->links.file.e);
		if (pos > 0
		    && (!a->extr || pos < a->lname)
		    && (!Extension || strcasecmp(Extension, a->links.file.e + pos + 1) == 0))
			{
			a->extr = x;
			a->lname = pos;
			if (Extension)
				return;
			}
		}
	}

void add_extrs(const char *path)
	{
	size_t lpath;
	DIR *dir;
	struct dirent *ent;
	struct extractor *x;
	char *abs, *file;

	dir = opendir(path);
	if (!dir)
		return;

	lpath = strlen(path);

	while ((ent = readdir(dir)))
		{
		if (is_dot(ent->d_name))
			continue;

		x = alloc(sizeof(struct extractor));

		abs = alloc(lpath + 1 + strlen(ent->d_name) + sizeof "/subdirectory");
		file = str_cp(str_cp(str_cp(abs, path), "("), ent->d_name);

		str_cp(file, ")$");
		if (regcomp(&x->matcher, abs + lpath, REG_EXTENDED | REG_ICASE) != 0)
			{
			err(MSG_REGCOMP, true, abs + lpath);
			free(x);
			free(abs);
			continue;
			}

		abs[lpath] = '/';

		str_cp(file, "/permissions");
		x->perms = file_type(true, abs) > Dir;
		str_cp(file, "/subdirectory");
		x->subdir = file_type(true, abs) > Dir;
		str_cp(file, "/extract");
		x->extract = abs;

		x->next = Extractors;
		Extractors = x;
		}

	closedir(dir);
	}
